/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file           : main.c
  * @brief          : Main program body for STM32F746ZG. The code is a
  *                   supplementary material to the article "Optimization for 
  *                   Software Implementation of Fractional Calculus Numerical 
  *                   Methods in an Embedded System". Peripherals initialization, 
  *                   clock configuration and linking with HAL libraries was made 
  *                   using STM32CubeMX For Eclipse plugin v.5.1.0 by 
  *                   STMicroelectronics.
  * @author         : Mariusz Matusiak <mmatusiak@iis.p.lodz.pl>
  *                   Lodz University of Technology,
  *                   18/22 Stefanowskiego St., Lodz, Poland
  ******************************************************************************
  ** This notice applies to any and all portions of this file
  * that are not between comment pairs USER CODE BEGIN and
  * USER CODE END. Other portions of this file, whether
  * inserted by the user or by software development tools
  * are owned by their respective copyright owners.
  *
  * COPYRIGHT(c) 2019 STMicroelectronics
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
  *   1. Redistributions of source code must retain the above copyright notice,
  *      this list of conditions and the following disclaimer.
  *   2. Redistributions in binary form must reproduce the above copyright notice,
  *      this list of conditions and the following disclaimer in the documentation
  *      and/or other materials provided with the distribution.
  *   3. Neither the name of STMicroelectronics nor the names of its contributors
  *      may be used to endorse or promote products derived from this software
  *      without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */
/* USER CODE END Header */

/* Includes ------------------------------------------------------------------*/
#include "main.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include <stdint.h>
#include <stdlib.h>
#include <math.h>
#include <float.h>

/* Select the desired core (M7 supports hardware FPU) */
#define TARGET_ARM_CORTEX_M3 0
#define TARGET_ARM_CORTEX_M7 1

/* Using arm_cortex_M3l_math/arm_cortex_M7lfsp_math libs from CMSIS-DSP */
#define OPTIMIZED
#ifdef OPTIMIZED
    #if TARGET_ARM_CORTEX_M7
        #define ARM_MATH_CM7
        #define __FPU_PRESENT 1
        #define __DSP_PRESENT 1
        #define __FPU_USED 1
    #endif
    #if TARGET_ARM_CORTEX_M3
        #define ARM_MATH_CM3
        #define __FPU_PRESENT 0
    #endif
    #include <arm_math.h>
#endif
/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */
typedef uint8_t bool;
/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */
/* Constants */
#define TEST_ARRAYS_PERF_SIZE_FULL  256
#define TEST_ARRAYS_PERF_SIZE_MED   255
#define TEST_ARRAYS_PERF_SIZE_SHORT 32

/* Test modes - enable (1) only one at a time */
#define TEST_FOD                1
#define TEST_VFOD               0
#define TEST_OPTIMIZED_VFOD     0
#define TEST_FIXED_POINT_VFOD   0
#define Q_M 11
#define Q_N 21
/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */
/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/
ADC_HandleTypeDef hadc1;

DAC_HandleTypeDef hdac;

TIM_HandleTypeDef htim1;
TIM_HandleTypeDef htim4;
TIM_HandleTypeDef htim10;
TIM_HandleTypeDef htim11;

UART_HandleTypeDef huart1;
UART_HandleTypeDef huart3;

/* USER CODE BEGIN PV */
/* Private variables ---------------------------------------------------------*/
volatile bool is_busy = 0; /* busy state indicator */

/* DWT unit registers */
volatile uint32_t *DWT_CONTROL = (uint32_t *)0xE0001000;
volatile uint32_t *DWT_CYCCNT =  (uint32_t *)0xE0001004;
volatile uint32_t *DEMCR =       (uint32_t *)0xE000EDFC;
volatile uint32_t *LAR  =        (uint32_t *)0xE0001FB0;

/* Variables for numbers of CPU cycles */
volatile uint32_t cpu_cycles;
volatile uint32_t cpu_cycles2;
volatile uint32_t cpu_cycles3;
/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
static void MX_GPIO_Init(void);
static void MX_TIM10_Init(void);
static void MX_ADC1_Init(void);
static void MX_USART1_UART_Init(void);
static void MX_TIM11_Init(void);
static void MX_DAC_Init(void);
static void MX_TIM1_Init(void);
static void MX_TIM4_Init(void);
static void MX_USART3_UART_Init(void);
/* USER CODE BEGIN PFP */
/* Private function prototypes -----------------------------------------------*/
void setBusyState(bool enabled);

static inline void resetArrays(float * array, int size, float value);

static inline void resetArrays_q31(q31_t * array, int size, q31_t value);

static inline void getLinSignal(float * array, int size);

float calculateACoeffsRecursively_float(const float v, unsigned int i, float * a_params_array);

q31_t calculateACoeffs_q31(const q31_t v, const uint32_t size, q31_t * a_params_array);

float calculateFractionalDerivativeStandard_float(const uint32_t n_samples, float * a_params, float * inputs, float * outputs, const float v, const float h);

q31_t calculateFractionalDerivativeStandard_q31(const uint32_t n_samples, q31_t * a_params, q31_t * inputs, q31_t * outputs, const q31_t denominator);

float32_t calculateFractionalDerivativeOptimized_float(const uint32_t n_samples, float32_t * a_params, float32_t * inputs, float32_t * outputs, const float32_t denominator);

void float32_to_Qm_n(float32_t * x_input, q31_t * x_input_q31, const uint32_t array_size, const uint16_t Qn);

void Qm_n_to_float32(q31_t * x_input_optimized_q31, float32_t * x_input, const uint32_t array_size, const uint16_t Qn);

extern void initialise_monitor_handles(void);
/* USER CODE END PFP */

/* Private user code ---------------------------------------------------------*/
/* USER CODE BEGIN 0 */
/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  * @retval int
  */
int main(void)
{
  /* USER CODE BEGIN 1 */
  /* USER CODE END 1 */

  /* MCU Configuration--------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* USER CODE BEGIN Init */
  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

  /* USER CODE BEGIN SysInit */
  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_TIM10_Init();
  MX_ADC1_Init();
  MX_USART1_UART_Init();
  MX_TIM11_Init();
  MX_DAC_Init();
  MX_TIM1_Init();
  MX_TIM4_Init();
  MX_USART3_UART_Init();
  /* USER CODE BEGIN 2 */

  /* Set GPIO output pins */
  HAL_GPIO_WritePin(GPIOB, LD2_LED_BLUE_Pin, GPIO_PIN_SET);
  HAL_GPIO_WritePin(GPIOB, LD3_LED_RED_Pin, GPIO_PIN_RESET);

  /* Enable DWT unit */
  *DEMCR = *DEMCR | 0x01000000;     // enable DWT
  *LAR = 0xC5ACCE55;                // enable access to register
  *DWT_CYCCNT = 0;                  // reset cycle counter
  *DWT_CONTROL = *DWT_CONTROL | 1;  // enable cycle counter

  /* Initialize test variables and arrays */
  const float h_sampling_time = 0.5f;
  float v_order = 0.7f;

#if TEST_VFOD || TEST_FOD
  float           x_input[TEST_ARRAYS_PERF_SIZE_FULL];
  volatile float y_output[TEST_ARRAYS_PERF_SIZE_FULL];
  volatile float a_params[TEST_ARRAYS_PERF_SIZE_FULL];
#endif

#if TEST_OPTIMIZED_VFOD
  float32_t           x_input_optimized[TEST_ARRAYS_PERF_SIZE_FULL];
  volatile float32_t y_output_optimized[TEST_ARRAYS_PERF_SIZE_FULL];
  volatile float32_t a_params_optimized[TEST_ARRAYS_PERF_SIZE_FULL];
#endif

#if TEST_FIXED_POINT_VFOD
  float32_t             x_input_optimized[TEST_ARRAYS_PERF_SIZE_FULL];
  q31_t             x_input_optimized_q31[TEST_ARRAYS_PERF_SIZE_FULL];
  volatile q31_t    y_output_q31[TEST_ARRAYS_PERF_SIZE_FULL];
  volatile q31_t    a_params_q31[TEST_ARRAYS_PERF_SIZE_FULL];
  q31_t v_order_q31 = (q31_t)(roundf(v_order * (1 << Q_N)));
#endif

  /* 1.1. fill input testing values */
#if TEST_VFOD || TEST_FOD
  //resetArrays(x_input, TEST_ARRAYS_PERF_SIZE_FULL, 1.0f);
  getLinSignal(x_input, TEST_ARRAYS_PERF_SIZE_FULL);
  resetArrays(y_output, TEST_ARRAYS_PERF_SIZE_FULL, 0.0f);
  resetArrays(a_params, TEST_ARRAYS_PERF_SIZE_FULL, 0.0f);
#endif

  /* 1.2. precompute a_j^\nu coeffs for FOD test */
#if TEST_FOD
  calculateACoeffsRecursively_float(v_order, TEST_ARRAYS_PERF_SIZE_FULL-1, a_params);
#endif

#if TEST_OPTIMIZED_VFOD
  resetArrays(x_input_optimized,  TEST_ARRAYS_PERF_SIZE_FULL, 1.0f);
  resetArrays(y_output_optimized, TEST_ARRAYS_PERF_SIZE_FULL, 0.0f);
  resetArrays(a_params_optimized, TEST_ARRAYS_PERF_SIZE_FULL, 0.0f);
#endif

#if TEST_FIXED_POINT_VFOD
  //resetArrays(x_input_optimized, TEST_ARRAYS_PERF_SIZE_FULL, 1.0f);
  getLinSignal(x_input_optimized, TEST_ARRAYS_PERF_SIZE_FULL);
  float32_to_Qm_n(x_input_optimized, x_input_optimized_q31, TEST_ARRAYS_PERF_SIZE_FULL,  Q_N);
  resetArrays_q31(a_params_q31, TEST_ARRAYS_PERF_SIZE_FULL, 0);
  resetArrays_q31(y_output_q31, TEST_ARRAYS_PERF_SIZE_FULL, 0);
#endif

#if TEST_OPTIMIZED_VFOD || TEST_FIXED_POINT_VFOD
  const float32_t denominator = 1.0f / powf(h_sampling_time, v_order);
#endif

#if TEST_FIXED_POINT_VFOD
  const q31_t denominator_q31 = (q31_t)(roundf(denominator * (1 << Q_N)));
#endif
  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */
  while (1)
  {
#if TEST_FOD
    *DWT_CYCCNT = 0;
    /* 2.1. calculate in standard way 64 samples */
    setBusyState(1);
    calculateFractionalDerivativeStandard_float(TEST_ARRAYS_PERF_SIZE_SHORT, a_params, x_input, y_output, v_order, h_sampling_time);
    setBusyState(0);
    cpu_cycles = *DWT_CYCCNT;
    resetArrays(y_output, TEST_ARRAYS_PERF_SIZE_FULL, 0.0f);
    /* 2.2. calculate in standard way 255 samples */
    *DWT_CYCCNT = 0;
    setBusyState(1);
    calculateFractionalDerivativeStandard_float(TEST_ARRAYS_PERF_SIZE_MED, a_params, x_input, y_output, v_order, h_sampling_time);
    setBusyState(0);
    cpu_cycles2 = *DWT_CYCCNT;
    resetArrays(y_output, TEST_ARRAYS_PERF_SIZE_FULL, 0.0f);
    /* 2.3. calculate in standard way 256 samples */
    *DWT_CYCCNT = 0;
    setBusyState(1);
    calculateFractionalDerivativeStandard_float(TEST_ARRAYS_PERF_SIZE_FULL, a_params, x_input, y_output, v_order, h_sampling_time);
    setBusyState(0);
    cpu_cycles3 = *DWT_CYCCNT;
    resetArrays(y_output, TEST_ARRAYS_PERF_SIZE_FULL, 0.0f);
#endif
#if TEST_VFOD
    /* 2.1. change the value of v_order */
    v_order += 10e-6f;
    *DWT_CYCCNT = 0;
    /* 2.2. calculate in standard way 64 samples of VFOD */
    setBusyState(1);
    calculateACoeffsRecursively_float(v_order, TEST_ARRAYS_PERF_SIZE_SHORT-1, a_params);
    calculateFractionalDerivativeStandard_float(TEST_ARRAYS_PERF_SIZE_SHORT, a_params, x_input, y_output, v_order, h_sampling_time);
    setBusyState(0);
    cpu_cycles = *DWT_CYCCNT;
    resetArrays(y_output, TEST_ARRAYS_PERF_SIZE_FULL, 0.0f);
    resetArrays(a_params, TEST_ARRAYS_PERF_SIZE_FULL, 0.0f);
    /* 2.3. calculate in standard way 255 samples of VFOD */
    *DWT_CYCCNT = 0;
    setBusyState(1);
    calculateACoeffsRecursively_float(v_order, TEST_ARRAYS_PERF_SIZE_MED-1, a_params);
    calculateFractionalDerivativeStandard_float(TEST_ARRAYS_PERF_SIZE_MED, a_params, x_input, y_output, v_order, h_sampling_time);
    setBusyState(0);
    cpu_cycles2 = *DWT_CYCCNT;
    resetArrays(y_output, TEST_ARRAYS_PERF_SIZE_FULL, 0.0f);
    resetArrays(a_params, TEST_ARRAYS_PERF_SIZE_FULL, 0.0f);
    /* 2.4. calculate in standard way 256 samples of VFOD */
    *DWT_CYCCNT = 0;
    setBusyState(1);
    calculateACoeffsRecursively_float(v_order, TEST_ARRAYS_PERF_SIZE_FULL-1, a_params);
    calculateFractionalDerivativeStandard_float(TEST_ARRAYS_PERF_SIZE_FULL, a_params, x_input, y_output, v_order, h_sampling_time);
    setBusyState(0);
    cpu_cycles3 = *DWT_CYCCNT;
    resetArrays(y_output, TEST_ARRAYS_PERF_SIZE_FULL, 0.0f);
    resetArrays(a_params, TEST_ARRAYS_PERF_SIZE_FULL, 0.0f);
#endif
#if TEST_OPTIMIZED_VFOD
    /* 2.1. change the value of v_order */
    v_order += 10e-6f;
    *DWT_CYCCNT = 0;
    /* 2.2. calculate in optimized way 64 samples of VFOD */
    setBusyState(1);
    calculateACoeffsRecursively_float(v_order, TEST_ARRAYS_PERF_SIZE_SHORT-1, a_params_optimized);
    calculateFractionalDerivativeOptimized_float(TEST_ARRAYS_PERF_SIZE_SHORT, a_params_optimized, x_input_optimized, y_output_optimized, denominator);
    setBusyState(0);
    cpu_cycles = *DWT_CYCCNT;
    resetArrays(y_output_optimized, TEST_ARRAYS_PERF_SIZE_FULL, 0.0f);
    resetArrays(a_params_optimized, TEST_ARRAYS_PERF_SIZE_FULL, 0.0f);
    *DWT_CYCCNT = 0;
    /* 2.3. calculate in optimized way 255 samples of VFOD */
    setBusyState(1);
    calculateACoeffsRecursively_float(v_order, TEST_ARRAYS_PERF_SIZE_MED-1, a_params_optimized);
    calculateFractionalDerivativeOptimized_float(TEST_ARRAYS_PERF_SIZE_MED, a_params_optimized, x_input_optimized, y_output_optimized, denominator);
    setBusyState(0);
    cpu_cycles2 = *DWT_CYCCNT;
    resetArrays(y_output_optimized, TEST_ARRAYS_PERF_SIZE_FULL, 0.0f);
    resetArrays(a_params_optimized, TEST_ARRAYS_PERF_SIZE_FULL, 0.0f);
    *DWT_CYCCNT = 0;
    /* 2.4. calculate in optimized way 256 samples of VFOD */
    setBusyState(1);
    calculateACoeffsRecursively_float(v_order, TEST_ARRAYS_PERF_SIZE_FULL-1, a_params_optimized);
    calculateFractionalDerivativeOptimized_float(TEST_ARRAYS_PERF_SIZE_FULL, a_params_optimized, x_input_optimized, y_output_optimized, denominator);
    setBusyState(0);
    cpu_cycles3 = *DWT_CYCCNT;
    resetArrays(y_output_optimized, TEST_ARRAYS_PERF_SIZE_FULL, 0.0f);
    resetArrays(a_params_optimized, TEST_ARRAYS_PERF_SIZE_FULL, 0.0f);
#endif
#if TEST_FIXED_POINT_VFOD
    /* 2.1. change the value of v_order_q31 */
    ++v_order_q31;
    *DWT_CYCCNT = 0;
    /* 2.2. calculate in fixed-point arithmetic 64 samples of VFOD */
    setBusyState(1);
    calculateACoeffs_q31(v_order_q31, TEST_ARRAYS_PERF_SIZE_SHORT, a_params_q31);
    calculateFractionalDerivativeStandard_q31(TEST_ARRAYS_PERF_SIZE_SHORT, a_params_q31, x_input_optimized_q31, y_output_q31, denominator_q31);
    setBusyState(0);
    cpu_cycles = *DWT_CYCCNT;
    resetArrays_q31(a_params_q31, TEST_ARRAYS_PERF_SIZE_FULL, 0);
    resetArrays_q31(y_output_q31, TEST_ARRAYS_PERF_SIZE_FULL, 0);
    *DWT_CYCCNT = 0;
    /* 2.3. calculate in fixed-point arithmetic 255 samples of VFOD */
    setBusyState(1);
    calculateACoeffs_q31(v_order_q31, TEST_ARRAYS_PERF_SIZE_MED, a_params_q31);
    calculateFractionalDerivativeStandard_q31(TEST_ARRAYS_PERF_SIZE_MED, a_params_q31, x_input_optimized_q31, y_output_q31, denominator_q31);
    setBusyState(0);
    cpu_cycles2 = *DWT_CYCCNT;
    resetArrays_q31(a_params_q31, TEST_ARRAYS_PERF_SIZE_FULL, 0);
    resetArrays_q31(y_output_q31, TEST_ARRAYS_PERF_SIZE_FULL, 0);
    *DWT_CYCCNT = 0;
    /* 2.4. calculate in fixed-point arithmetic way 256 samples of VFOD */
    setBusyState(1);
    calculateACoeffs_q31(v_order_q31, TEST_ARRAYS_PERF_SIZE_FULL, a_params_q31);
    calculateFractionalDerivativeStandard_q31(TEST_ARRAYS_PERF_SIZE_FULL, a_params_q31, x_input_optimized_q31, y_output_q31, denominator_q31);
    setBusyState(0);
    cpu_cycles3 = *DWT_CYCCNT;
    Qm_n_to_float32(y_output_q31, x_input_optimized, TEST_ARRAYS_PERF_SIZE_FULL, Q_N);
    resetArrays_q31(a_params_q31, TEST_ARRAYS_PERF_SIZE_FULL, 0);
    resetArrays_q31(y_output_q31, TEST_ARRAYS_PERF_SIZE_FULL, 0);
#endif
    /* USER CODE END WHILE */

    /* USER CODE BEGIN 3 */
  }
  /* USER CODE END 3 */
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};
  RCC_PeriphCLKInitTypeDef PeriphClkInitStruct = {0};

  /** Configure the main internal regulator output voltage 
  */
  __HAL_RCC_PWR_CLK_ENABLE();
  __HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE1);
  /** Initializes the CPU, AHB and APB busses clocks 
  */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSI;
  RCC_OscInitStruct.HSIState = RCC_HSI_ON;
  RCC_OscInitStruct.HSICalibrationValue = RCC_HSICALIBRATION_DEFAULT;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSI;
  RCC_OscInitStruct.PLL.PLLM = 8;
  RCC_OscInitStruct.PLL.PLLN = 216;
  RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV2;
  RCC_OscInitStruct.PLL.PLLQ = 2;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }
  /** Activate the Over-Drive mode 
  */
  if (HAL_PWREx_EnableOverDrive() != HAL_OK)
  {
    Error_Handler();
  }
  /** Initializes the CPU, AHB and APB busses clocks 
  */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV4;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV2;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_7) != HAL_OK)
  {
    Error_Handler();
  }
  PeriphClkInitStruct.PeriphClockSelection = RCC_PERIPHCLK_USART1|RCC_PERIPHCLK_USART3;
  PeriphClkInitStruct.Usart1ClockSelection = RCC_USART1CLKSOURCE_PCLK2;
  PeriphClkInitStruct.Usart3ClockSelection = RCC_USART3CLKSOURCE_PCLK1;
  if (HAL_RCCEx_PeriphCLKConfig(&PeriphClkInitStruct) != HAL_OK)
  {
    Error_Handler();
  }
}

/**
  * @brief ADC1 Initialization Function
  * @param None
  * @retval None
  */
static void MX_ADC1_Init(void)
{

  /* USER CODE BEGIN ADC1_Init 0 */

  /* USER CODE END ADC1_Init 0 */

  ADC_ChannelConfTypeDef sConfig = {0};

  /* USER CODE BEGIN ADC1_Init 1 */

  /* USER CODE END ADC1_Init 1 */
  /** Configure the global features of the ADC (Clock, Resolution, Data Alignment and number of conversion) 
  */
  hadc1.Instance = ADC1;
  hadc1.Init.ClockPrescaler = ADC_CLOCK_SYNC_PCLK_DIV8;
  hadc1.Init.Resolution = ADC_RESOLUTION_12B;
  hadc1.Init.ScanConvMode = ADC_SCAN_DISABLE;
  hadc1.Init.ContinuousConvMode = DISABLE;
  hadc1.Init.DiscontinuousConvMode = DISABLE;
  hadc1.Init.ExternalTrigConvEdge = ADC_EXTERNALTRIGCONVEDGE_NONE;
  hadc1.Init.ExternalTrigConv = ADC_SOFTWARE_START;
  hadc1.Init.DataAlign = ADC_DATAALIGN_RIGHT;
  hadc1.Init.NbrOfConversion = 1;
  hadc1.Init.DMAContinuousRequests = DISABLE;
  hadc1.Init.EOCSelection = ADC_EOC_SINGLE_CONV;
  if (HAL_ADC_Init(&hadc1) != HAL_OK)
  {
    Error_Handler();
  }
  /** Configure for the selected ADC regular channel its corresponding rank in the sequencer and its sample time. 
  */
  sConfig.Channel = ADC_CHANNEL_11;
  sConfig.Rank = ADC_REGULAR_RANK_1;
  sConfig.SamplingTime = ADC_SAMPLETIME_56CYCLES;
  if (HAL_ADC_ConfigChannel(&hadc1, &sConfig) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN ADC1_Init 2 */

  /* USER CODE END ADC1_Init 2 */

}

/**
  * @brief DAC Initialization Function
  * @param None
  * @retval None
  */
static void MX_DAC_Init(void)
{

  /* USER CODE BEGIN DAC_Init 0 */

  /* USER CODE END DAC_Init 0 */

  DAC_ChannelConfTypeDef sConfig = {0};

  /* USER CODE BEGIN DAC_Init 1 */

  /* USER CODE END DAC_Init 1 */
  /** DAC Initialization 
  */
  hdac.Instance = DAC;
  if (HAL_DAC_Init(&hdac) != HAL_OK)
  {
    Error_Handler();
  }
  /** DAC channel OUT2 config 
  */
  sConfig.DAC_Trigger = DAC_TRIGGER_NONE;
  sConfig.DAC_OutputBuffer = DAC_OUTPUTBUFFER_DISABLE;
  if (HAL_DAC_ConfigChannel(&hdac, &sConfig, DAC_CHANNEL_2) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN DAC_Init 2 */

  /* USER CODE END DAC_Init 2 */

}

/**
  * @brief TIM1 Initialization Function
  * @param None
  * @retval None
  */
static void MX_TIM1_Init(void)
{

  /* USER CODE BEGIN TIM1_Init 0 */

  /* USER CODE END TIM1_Init 0 */

  TIM_Encoder_InitTypeDef sConfig = {0};
  TIM_MasterConfigTypeDef sMasterConfig = {0};

  /* USER CODE BEGIN TIM1_Init 1 */

  /* USER CODE END TIM1_Init 1 */
  htim1.Instance = TIM1;
  htim1.Init.Prescaler = 0;
  htim1.Init.CounterMode = TIM_COUNTERMODE_UP;
  htim1.Init.Period = 8000;
  htim1.Init.ClockDivision = TIM_CLOCKDIVISION_DIV1;
  htim1.Init.RepetitionCounter = 0;
  htim1.Init.AutoReloadPreload = TIM_AUTORELOAD_PRELOAD_DISABLE;
  sConfig.EncoderMode = TIM_ENCODERMODE_TI12;
  sConfig.IC1Polarity = TIM_ICPOLARITY_RISING;
  sConfig.IC1Selection = TIM_ICSELECTION_DIRECTTI;
  sConfig.IC1Prescaler = TIM_ICPSC_DIV1;
  sConfig.IC1Filter = 15;
  sConfig.IC2Polarity = TIM_ICPOLARITY_RISING;
  sConfig.IC2Selection = TIM_ICSELECTION_DIRECTTI;
  sConfig.IC2Prescaler = TIM_ICPSC_DIV1;
  sConfig.IC2Filter = 15;
  if (HAL_TIM_Encoder_Init(&htim1, &sConfig) != HAL_OK)
  {
    Error_Handler();
  }
  sMasterConfig.MasterOutputTrigger = TIM_TRGO_RESET;
  sMasterConfig.MasterOutputTrigger2 = TIM_TRGO2_RESET;
  sMasterConfig.MasterSlaveMode = TIM_MASTERSLAVEMODE_DISABLE;
  if (HAL_TIMEx_MasterConfigSynchronization(&htim1, &sMasterConfig) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN TIM1_Init 2 */

  /* USER CODE END TIM1_Init 2 */

}

/**
  * @brief TIM4 Initialization Function
  * @param None
  * @retval None
  */
static void MX_TIM4_Init(void)
{

  /* USER CODE BEGIN TIM4_Init 0 */

  /* USER CODE END TIM4_Init 0 */

  TIM_ClockConfigTypeDef sClockSourceConfig = {0};
  TIM_MasterConfigTypeDef sMasterConfig = {0};
  TIM_OC_InitTypeDef sConfigOC = {0};

  /* USER CODE BEGIN TIM4_Init 1 */

  /* USER CODE END TIM4_Init 1 */
  htim4.Instance = TIM4;
  htim4.Init.Prescaler = 71;
  htim4.Init.CounterMode = TIM_COUNTERMODE_UP;
  htim4.Init.Period = 49999;
  htim4.Init.ClockDivision = TIM_CLOCKDIVISION_DIV1;
  htim4.Init.AutoReloadPreload = TIM_AUTORELOAD_PRELOAD_ENABLE;
  if (HAL_TIM_Base_Init(&htim4) != HAL_OK)
  {
    Error_Handler();
  }
  sClockSourceConfig.ClockSource = TIM_CLOCKSOURCE_INTERNAL;
  if (HAL_TIM_ConfigClockSource(&htim4, &sClockSourceConfig) != HAL_OK)
  {
    Error_Handler();
  }
  if (HAL_TIM_PWM_Init(&htim4) != HAL_OK)
  {
    Error_Handler();
  }
  sMasterConfig.MasterOutputTrigger = TIM_TRGO_RESET;
  sMasterConfig.MasterSlaveMode = TIM_MASTERSLAVEMODE_DISABLE;
  if (HAL_TIMEx_MasterConfigSynchronization(&htim4, &sMasterConfig) != HAL_OK)
  {
    Error_Handler();
  }
  sConfigOC.OCMode = TIM_OCMODE_PWM1;
  sConfigOC.Pulse = 0;
  sConfigOC.OCPolarity = TIM_OCPOLARITY_HIGH;
  sConfigOC.OCFastMode = TIM_OCFAST_DISABLE;
  if (HAL_TIM_PWM_ConfigChannel(&htim4, &sConfigOC, TIM_CHANNEL_3) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN TIM4_Init 2 */

  /* USER CODE END TIM4_Init 2 */
  HAL_TIM_MspPostInit(&htim4);

}

/**
  * @brief TIM10 Initialization Function
  * @param None
  * @retval None
  */
static void MX_TIM10_Init(void)
{

  /* USER CODE BEGIN TIM10_Init 0 */

  /* USER CODE END TIM10_Init 0 */

  /* USER CODE BEGIN TIM10_Init 1 */

  /* USER CODE END TIM10_Init 1 */
  htim10.Instance = TIM10;
  htim10.Init.Prescaler = 43199;
  htim10.Init.CounterMode = TIM_COUNTERMODE_UP;
  htim10.Init.Period = 14999;
  htim10.Init.ClockDivision = TIM_CLOCKDIVISION_DIV1;
  htim10.Init.AutoReloadPreload = TIM_AUTORELOAD_PRELOAD_ENABLE;
  if (HAL_TIM_Base_Init(&htim10) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN TIM10_Init 2 */

  /* USER CODE END TIM10_Init 2 */

}

/**
  * @brief TIM11 Initialization Function
  * @param None
  * @retval None
  */
static void MX_TIM11_Init(void)
{

  /* USER CODE BEGIN TIM11_Init 0 */

  /* USER CODE END TIM11_Init 0 */

  /* USER CODE BEGIN TIM11_Init 1 */

  /* USER CODE END TIM11_Init 1 */
  htim11.Instance = TIM11;
  htim11.Init.Prescaler = 215;
  htim11.Init.CounterMode = TIM_COUNTERMODE_UP;
  htim11.Init.Period = 9999;
  htim11.Init.ClockDivision = TIM_CLOCKDIVISION_DIV1;
  htim11.Init.AutoReloadPreload = TIM_AUTORELOAD_PRELOAD_ENABLE;
  if (HAL_TIM_Base_Init(&htim11) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN TIM11_Init 2 */

  /* USER CODE END TIM11_Init 2 */

}

/**
  * @brief USART1 Initialization Function
  * @param None
  * @retval None
  */
static void MX_USART1_UART_Init(void)
{

  /* USER CODE BEGIN USART1_Init 0 */

  /* USER CODE END USART1_Init 0 */

  /* USER CODE BEGIN USART1_Init 1 */

  /* USER CODE END USART1_Init 1 */
  huart1.Instance = USART1;
  huart1.Init.BaudRate = 926000;
  huart1.Init.WordLength = UART_WORDLENGTH_8B;
  huart1.Init.StopBits = UART_STOPBITS_1;
  huart1.Init.Parity = UART_PARITY_NONE;
  huart1.Init.Mode = UART_MODE_TX_RX;
  huart1.Init.HwFlowCtl = UART_HWCONTROL_NONE;
  huart1.Init.OverSampling = UART_OVERSAMPLING_16;
  huart1.Init.OneBitSampling = UART_ONE_BIT_SAMPLE_DISABLE;
  huart1.AdvancedInit.AdvFeatureInit = UART_ADVFEATURE_NO_INIT;
  if (HAL_UART_Init(&huart1) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN USART1_Init 2 */

  /* USER CODE END USART1_Init 2 */

}

/**
  * @brief USART3 Initialization Function
  * @param None
  * @retval None
  */
static void MX_USART3_UART_Init(void)
{

  /* USER CODE BEGIN USART3_Init 0 */

  /* USER CODE END USART3_Init 0 */

  /* USER CODE BEGIN USART3_Init 1 */

  /* USER CODE END USART3_Init 1 */
  huart3.Instance = USART3;
  huart3.Init.BaudRate = 926000;
  huart3.Init.WordLength = UART_WORDLENGTH_8B;
  huart3.Init.StopBits = UART_STOPBITS_1;
  huart3.Init.Parity = UART_PARITY_NONE;
  huart3.Init.Mode = UART_MODE_TX_RX;
  huart3.Init.HwFlowCtl = UART_HWCONTROL_NONE;
  huart3.Init.OverSampling = UART_OVERSAMPLING_16;
  huart3.Init.OneBitSampling = UART_ONE_BIT_SAMPLE_DISABLE;
  huart3.AdvancedInit.AdvFeatureInit = UART_ADVFEATURE_NO_INIT;
  if (HAL_UART_Init(&huart3) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN USART3_Init 2 */

  /* USER CODE END USART3_Init 2 */

}

/**
  * @brief GPIO Initialization Function
  * @param None
  * @retval None
  */
static void MX_GPIO_Init(void)
{
  GPIO_InitTypeDef GPIO_InitStruct = {0};

  /* GPIO Ports Clock Enable */
  __HAL_RCC_GPIOC_CLK_ENABLE();
  __HAL_RCC_GPIOA_CLK_ENABLE();
  __HAL_RCC_GPIOE_CLK_ENABLE();
  __HAL_RCC_GPIOB_CLK_ENABLE();
  __HAL_RCC_GPIOD_CLK_ENABLE();

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIO_OUTPUT_TEST_SIGNAL_GPIO_Port, GPIO_OUTPUT_TEST_SIGNAL_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOB, LD3_LED_RED_Pin|LD2_LED_BLUE_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(SEMAPHORE_GPIO_Port, SEMAPHORE_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pin : GPIO_BUTTON_INTERRUPT_PIN_Pin */
  GPIO_InitStruct.Pin = GPIO_BUTTON_INTERRUPT_PIN_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_IT_FALLING;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(GPIO_BUTTON_INTERRUPT_PIN_GPIO_Port, &GPIO_InitStruct);

  /*Configure GPIO pins : GPIO_ENCODER_INPUT_1_Pin GPIO_ENCODER_INPUT_2_Pin */
  GPIO_InitStruct.Pin = GPIO_ENCODER_INPUT_1_Pin|GPIO_ENCODER_INPUT_2_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING;
  GPIO_InitStruct.Pull = GPIO_PULLUP;
  HAL_GPIO_Init(GPIOC, &GPIO_InitStruct);

  /*Configure GPIO pin : GPIO_OUTPUT_TEST_SIGNAL_Pin */
  GPIO_InitStruct.Pin = GPIO_OUTPUT_TEST_SIGNAL_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIO_OUTPUT_TEST_SIGNAL_GPIO_Port, &GPIO_InitStruct);

  /*Configure GPIO pins : LD3_LED_RED_Pin LD2_LED_BLUE_Pin */
  GPIO_InitStruct.Pin = LD3_LED_RED_Pin|LD2_LED_BLUE_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

  /*Configure GPIO pin : SEMAPHORE_Pin */
  GPIO_InitStruct.Pin = SEMAPHORE_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_VERY_HIGH;
  HAL_GPIO_Init(SEMAPHORE_GPIO_Port, &GPIO_InitStruct);

  /* EXTI interrupt init*/
  HAL_NVIC_SetPriority(EXTI2_IRQn, 0, 0);
  HAL_NVIC_EnableIRQ(EXTI2_IRQn);

  HAL_NVIC_SetPriority(EXTI3_IRQn, 0, 0);
  HAL_NVIC_EnableIRQ(EXTI3_IRQn);

  HAL_NVIC_SetPriority(EXTI15_10_IRQn, 0, 0);
  HAL_NVIC_EnableIRQ(EXTI15_10_IRQn);

}

/* USER CODE BEGIN 4 */
void setBusyState(bool enabled)
{
    if (enabled)
    {
        is_busy = 1;
        HAL_GPIO_WritePin(SEMAPHORE_GPIO_Port, SEMAPHORE_Pin, GPIO_PIN_SET);
    } else
    {
        is_busy = 0;
        HAL_GPIO_WritePin(SEMAPHORE_GPIO_Port, SEMAPHORE_Pin, GPIO_PIN_RESET);
    }
}

static inline void resetArrays(float * array, int size, float value)
{
    for (int i = 0; i<size; ++i)
    {
        array[i] = value;
    }
}

static inline void resetArrays_q31(q31_t * array, int size, q31_t value)
{
    for (int i = 0; i<size; ++i)
    {
        array[i] = value;
    }
}

static inline void getLinSignal(float * array, int size)
{
    float K = 1.0f/(float)size;
    for(int i = 0; i < size; ++i)
    {
        array[i] = (float)i * K;
    }
}

float calculateACoeffsRecursively_float(const float v, unsigned int i, float * a_params_array) {
    const float a_param_init_value = 1.0f;
    if (i == 0) {
        a_params_array[i] = a_param_init_value;
    } else {
        a_params_array[i] = calculateACoeffsRecursively_float(v, (i - 1), a_params_array) * (1.0f - ((v + 1.0f)/(float)i));
    }
    return a_params_array[i];
}

q31_t calculateACoeffs_q31(const q31_t v, const uint32_t size, q31_t * a_params_array) {
    q31_t previous_a_param; //a_{j-1}^v
    q63_t temp;
    q31_t factor; //(v+1)/j
    q63_t nominator_of_the_factor = (q63_t)(v + (1 << Q_N)) << Q_N; //(v+1) prepared for division
    a_params_array[0] = (1 << Q_N); //1.0f
    for (uint32_t i = 1; i < size; ++i)
    {
        previous_a_param = a_params_array[i-1];
        factor = (q31_t)(nominator_of_the_factor / (i * (1 << Q_N)));
        temp = ((q63_t)previous_a_param * factor) >> Q_N;
        a_params_array[i] = previous_a_param - (q31_t)temp;
    }
    return a_params_array[size-1];
}

float calculateFractionalDerivativeStandard_float(const uint32_t n_samples, float * a_params, float * inputs, float * outputs, const float v, const float h) {
    float backward_difference_for_n;
    float factor;
    float a_param_for_j;
    float input_value_k;
    if (n_samples == 0)
    {
        return 0;
    }
    else
    {
        factor = powf(h, v);
        if (factor == 0.0f)
            return FLT_MAX;
        factor = 1.0f/factor;
        /* we assume that n_samples <= length(inputs) */
        for (uint32_t n = 0; n < n_samples; ++n) {
            backward_difference_for_n = 0.0f;
            for (uint32_t j = 0; j <= n; ++j)
            {
                input_value_k = inputs[n - j];
                a_param_for_j = a_params[j];
                backward_difference_for_n += (a_param_for_j * input_value_k);
            }
            outputs[n] = backward_difference_for_n * factor;
        }
        return outputs[n_samples-1];
    }
}

float32_t calculateFractionalDerivativeOptimized_float(const uint32_t n_samples, float32_t * a_params, float32_t * inputs, float32_t * outputs, const float32_t denominator) {
    /* Using arm_cortex_M3l_math/arm_cortex_M7lfsp_math libs from CMSIS-DSP */
    arm_conv_partial_f32(a_params, n_samples, inputs, n_samples, outputs, 0, n_samples);
    arm_scale_f32(outputs, denominator, outputs, n_samples);
    return outputs[n_samples-1];
}

q31_t calculateFractionalDerivativeStandard_q31(const uint32_t n_samples, q31_t * a_params, q31_t * inputs, q31_t * outputs, const q31_t denominator) {
    q63_t backward_difference_for_n;
    q63_t tmp;
    q31_t a_param_for_j;
    q31_t input_value_k;
    if (n_samples == 0)
    {
        return 0;
    }
    else
    {
        for (uint32_t n = 0; n < n_samples; ++n)
        {
            backward_difference_for_n = 0;
            for (uint32_t j = 0; j <= n; ++j)
            {
                input_value_k = inputs[n - j];
                a_param_for_j = a_params[j];
                tmp = (q63_t)a_param_for_j * input_value_k;
                backward_difference_for_n += (tmp >> Q_N);
            }
            outputs[n] = (q31_t)((backward_difference_for_n * denominator) >> Q_N);

        }
        return outputs[n_samples-1];
    }
}

void float32_to_Qm_n(float32_t * x_input, q31_t * x_input_q31, const uint32_t array_size, const uint16_t Qn)
{
    /* Saturation checking in final implementation is recommended */
    uint32_t scale = (1 << Qn);
    for (uint32_t i = 0; i < array_size; ++i)
    {
        x_input_q31[i] = (q31_t)(roundf(x_input[i] * scale));
    }
}

void Qm_n_to_float32(q31_t * x_input_optimized_q31, float32_t * x_input, const uint32_t array_size, const uint16_t Qn)
{
    const float scale = (float)(1 << Qn);
    for (uint32_t i = 0; i < array_size; ++i)
    {
        x_input[i] = (float)x_input_optimized_q31[i] / scale;
    }
}
/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */
  while(1)
  {
  }
  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{ 
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     tex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
